"use strict";
var ak_editor_prosemirror_1 = require("ak-editor-prosemirror");
var ak_editor_schema_1 = require("ak-editor-schema");
var matches_1 = require("./matches");
/**
 * ProseMirror doesn't support empty text nodes, which can be quite
 * inconvenient when you want to capture a position ref without introducing
 * text.
 *
 * Take a couple of examples:
 *
 *     p('{<>}')
 *     p('Hello ', '{<>}', 'world!')
 *
 * After the ref syntax is stripped you're left with:
 *
 *     p('')
 *     p('Hello ', '', 'world!')
 *
 * This violates the rule of text nodes being non-empty. This class solves the
 * problem by providing an alternative data structure that *only* stores refs,
 * and can be used in scenarios where an empty text would be forbidden.
 *
 * This is done under the hood when using `text()` factory, and instead of
 * always returning a text node, it'll instead return one of two things:
 *
 * - a text node -- when given a non-empty string
 * - a refs tracker -- when given a string that *only* contains refs.
 */
var RefsTracker = (function () {
    function RefsTracker() {
    }
    return RefsTracker;
}());
exports.RefsTracker = RefsTracker;
/**
 * Create a text node.
 *
 * Special markers called "refs" can be put in the text. Refs provide a way to
 * declaratively describe a position within some text, and then access the
 * position in the resulting node.
 */
function text(value, _schema) {
    if (_schema === void 0) { _schema = ak_editor_schema_1.default; }
    var stripped = "";
    var textIndex = 0;
    var refs = {};
    for (var _i = 0, _a = matches_1.default(value, /{(\w+|<|>|<>)}/g); _i < _a.length; _i++) {
        var match = _a[_i];
        var refToken = match[0], refName = match[1];
        stripped += value.slice(textIndex, match.index);
        refs[refName] = stripped.length;
        textIndex = match.index + refToken.length;
    }
    stripped += value.slice(textIndex);
    var node = stripped === ""
        ? new RefsTracker()
        : _schema.text(stripped);
    node.refs = refs;
    return node;
}
exports.text = text;
/**
 * Offset ref position values by some amount.
 */
function offsetRefs(refs, offset) {
    var result = {};
    for (var name_1 in refs) {
        result[name_1] = refs[name_1] + offset;
    }
    return result;
}
exports.offsetRefs = offsetRefs;
/**
 * Given a collection of nodes, sequence them in an array and return the result
 * along with the updated refs.
 */
function sequence() {
    var content = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        content[_i] = arguments[_i];
    }
    var position = 0;
    var refs = {};
    var nodes = [];
    // It's bizarre that this is necessary. An if/else in the for...of should have
    // sufficient but it did not work at the time of writing.
    var isRefsTracker = function (n) { return n instanceof RefsTracker; };
    var isRefsNode = function (n) { return !isRefsTracker(n); };
    for (var _a = 0, content_1 = content; _a < content_1.length; _a++) {
        var node = content_1[_a];
        if (isRefsTracker(node)) {
            Object.assign(refs, offsetRefs(node.refs, position));
        }
        if (isRefsNode(node)) {
            var thickness = node.isText ? 0 : 1;
            Object.assign(refs, offsetRefs(node.refs, position + thickness));
            position += node.nodeSize;
            nodes.push(node);
        }
    }
    return { nodes: nodes, refs: refs };
}
exports.sequence = sequence;
/**
 * Given a jagged array, flatten it down to a single level.
 */
function flatten(deep) {
    var flat = [];
    for (var _i = 0, deep_1 = deep; _i < deep_1.length; _i++) {
        var item = deep_1[_i];
        if (Array.isArray(item)) {
            flat.splice.apply(flat, [flat.length, 0].concat(item));
        }
        else {
            flat.push(item);
        }
    }
    return flat;
}
exports.flatten = flatten;
/**
 * Coerce builder content into ref nodes.
 */
function coerce(content, schema) {
    var refsContent = content
        .map(function (item) { return typeof item === 'string'
        ? text(item, schema)
        : item; });
    return sequence.apply(void 0, flatten(refsContent));
}
exports.coerce = coerce;
/**
 * Create a factory for nodes.
 */
function nodeFactory(type, attrs) {
    if (attrs === void 0) { attrs = {}; }
    return function () {
        var content = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            content[_i] = arguments[_i];
        }
        var _a = coerce(content, type.schema), nodes = _a.nodes, refs = _a.refs;
        var node = type.create(attrs, nodes);
        node.refs = refs;
        return node;
    };
}
exports.nodeFactory = nodeFactory;
/**
 * Create a factory for marks.
 */
function markFactory(type, attrs) {
    if (attrs === void 0) { attrs = {}; }
    var mark = type.create(attrs);
    return function () {
        var content = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            content[_i] = arguments[_i];
        }
        var nodes = coerce(content, type.schema).nodes;
        return nodes
            .map(function (node) {
            if (mark.type.isInSet(node.marks)) {
                return node;
            }
            else {
                var refNode = node.mark(mark.addToSet(node.marks));
                refNode.refs = node.refs;
                return refNode;
            }
        });
    };
}
exports.markFactory = markFactory;
;
exports.doc = nodeFactory(ak_editor_schema_1.default.nodes.doc, {});
exports.p = nodeFactory(ak_editor_schema_1.default.nodes.paragraph, {});
exports.blockquote = nodeFactory(ak_editor_schema_1.default.nodes.blockquote, {});
exports.h1 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 1 });
exports.h2 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 2 });
exports.h3 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 3 });
exports.h4 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 4 });
exports.h5 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 5 });
exports.h6 = nodeFactory(ak_editor_schema_1.default.nodes.heading, { level: 6 });
exports.li = nodeFactory(ak_editor_schema_1.default.nodes.list_item, {});
exports.ul = nodeFactory(ak_editor_schema_1.default.nodes.bullet_list, {});
exports.ol = nodeFactory(ak_editor_schema_1.default.nodes.ordered_list, {});
exports.br = ak_editor_schema_1.default.nodes.hard_break.createChecked();
exports.code_block = function (attrs) {
    if (attrs === void 0) { attrs = {}; }
    return nodeFactory(ak_editor_schema_1.default.nodes.code_block, attrs);
};
exports.img = function (attrs) { return ak_editor_schema_1.default.nodes.image.createChecked(attrs); };
exports.emoji = function (attrs) { return ak_editor_schema_1.default.nodes.emoji.createChecked(attrs); };
exports.mention = function (attrs) { return ak_editor_schema_1.default.nodes.mention.createChecked(attrs); };
exports.hr = ak_editor_schema_1.default.nodes.horizontal_rule.createChecked();
exports.em = markFactory(ak_editor_schema_1.default.marks.em, {});
exports.strong = markFactory(ak_editor_schema_1.default.marks.strong, {});
exports.code = markFactory(ak_editor_schema_1.default.marks.code, {});
exports.del = markFactory(ak_editor_schema_1.default.marks.del, {});
exports.a = function (attrs) { return markFactory(ak_editor_schema_1.default.marks.link, attrs); };
exports.fragment = function () {
    var content = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        content[_i] = arguments[_i];
    }
    return flatten(content);
};
exports.slice = function () {
    var content = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        content[_i] = arguments[_i];
    }
    return new ak_editor_prosemirror_1.Slice(new ak_editor_prosemirror_1.Fragment(flatten(content)), 0, 0);
};
//# sourceMappingURL=schema-builder.js.map